unit config_project;
{*
 * Filename.....: config_project.pas (dbknitofrontend)
 * Purpose......: config module for projects
 *
 * Erstellt am..: Dienstag, 24. Februar 2004, 12:17:14
 *       _  __      _ _
 *  ||| | |/ /     (_) |        Wirtschaftsinformatiker IHK
 * \. ./| ' / _ __  _| |_ ___   www.ingoknito.de
 * - ^ -|  < | '_ \| | __/ _ \
 * / - \| . \| | | | | || (_) | Peter Klauer
 *  ||| |_|\_\_| |_|_|\__\___/
 * mailto.......: knito@knito.de
 *
 * Changes:
 * 2004-03-04: Added spin-edit, deleted history tabsheet
 *}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, JvScrollBox, ComCtrls, mystring, ExtCtrls,
  Mask, JvToolEdit, inifiles, JvPlacemnt, menus, XPMan, JvPanel,
  JvMaskEdit, JvSpin, JvComponent, JvBalloonHint;

type
  Tfrm_project_config = class(TForm)
    StatusBar1: TStatusBar;
    PageControl: TPageControl;
    ts_general: TTabSheet;
    Panel1: TPanel;
    btn_readconfig: TBitBtn;
    btn_close: TBitBtn;
    JvFormStorage1: TJvFormStorage;
    btn_saveproject: TBitBtn;
    JvScrollBox1: TJvScrollBox;
    Label3: TLabel;
    Label35: TLabel;
    lbl_projectpath: TLabel;
    lbl_manualfile: TLabel;
    lbl_www: TLabel;
    edit_general_projectname: TEdit;
    edit_general_projectdir: TJvFilenameEdit;
    edit_general_snapshotimage: TJvFilenameEdit;
    cb_general_exit: TCheckBox;
    cb_general_noconsole: TCheckBox;
    edit_general_manualfile: TJvFilenameEdit;
    edit_general_www: TEdit;
    XPManifest1: TXPManifest;
    pnl_project_config: TJvPanel;
    Image3: TImage;
    JvBalloonHint1: TJvBalloonHint;
    procedure btn_readconfigClick(Sender: TObject);
    procedure btn_saveprojectClick(Sender: TObject);
    procedure edit_general_projectnameChange(Sender: TObject);
    procedure lbl_projectpathClick(Sender: TObject);
    procedure lbl_manualfileClick(Sender: TObject);
    procedure lbl_wwwClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure readdata( ini : tinifile );
    function str_quote_dir( s : string ) : string;    
  end;

var
  frm_project_config: Tfrm_project_config;

implementation

uses dbkf_main, config_dosbox;


{$R *.dfm}

procedure Tfrm_project_config.btn_readconfigClick(Sender: TObject);
var
  configfile, s, section, hint, eol : string;

  f : textfile;

  left, top, hoffset, voffset, i : integer;

  newtabsheet : ttabsheet;

  newlabel    : tlabel;
  newedit     : tedit;
  newcheckbox : tcheckbox;
  newcombobox : tcombobox;
  newmemo     : tmemo;
  newitem     : tmenuitem;
  newscrollbox: tjvscrollbox;

  newspinedit : tjvspinedit;

  sl : tstringlist;
  c1 : string; // firstchar
  first : boolean;

begin

  // one more which stays empty - takes
  // all items that anybody would configure
  // BEFORE the first section.
  // Never shown, but saves programs life.
  newscrollbox := tjvscrollbox.create( self );
  newtabsheet := ttabsheet.create( self );

  hint  := '';
  eol   := '';
  first := true; // first hint for #13#10 management

  sl := tstringlist.create();

  left := 8;
  top := 0;
  voffset := 25;
  hoffset := 100;

  configfile :=
    nolastbackslash(frm_dosbox_config.lbl_mypath.Caption) +
    '\'+ frm_dosbox_config.edit_configfile.text;

  if not fileexists( configfile ) then
  begin
    MessageDlg(
      'ERROR:'+#13+#10+
      'The configuration file '+#13+#10+
      configfile+#13+#10+
      'does not exist!'+#13+#10+
      'Please get it, put it into the directory and restart me!',
      mtError, [mbOK], 0);
    exit;
  end;

  AssignFile(F, configfile); { Datei in Dialogfeld ausgewhlt }
  Reset(F);

  while not eof( f ) do
  begin

    Readln(F, S);

    S := trim( S );

    if length( S ) = 0 then continue;

    c1 := copy( S, 1, 1 ); // firstchar



    if (c1 = '#') then
    begin
      if first = false then eol := #13#10;
      hint := hint + eol + alltrim( copy( s, 2, length( s ) - 1 ) );
      first := false;
      continue;
    end;

    if (c1 = ';') then continue;

    if c1 = '[' then
    begin

      top := 0;
      newtabsheet.Hint := hint;

      // reset hint status for next section
      hint  := '';
      first := true; // first hint row for #13#10 management
      eol   := '';

      newtabsheet := ttabsheet.create( self );
      section := trim( copy( s, 2, length( s )- 2  ) );
      newtabsheet.Caption := section;
      newtabsheet.name := 'ts_'+section;

      newtabsheet.parent := PageControl;
      newtabsheet.PageControl := PageControl;
      PageControl.Update;

      newscrollbox := tjvscrollbox.Create( self );
      newscrollbox.Parent := newtabsheet;
      newscrollbox.align := alClient;
      newscrollbox.BorderStyle := bsNone;
      newscrollbox.BevelInner := bvNone;
      newscrollbox.BevelKind := bkNone;
      newscrollbox.BevelOuter := bvNone;

      newitem := tmenuitem.Create( dbkf );
      newitem.Action := DBKF.act_opensection;
      newitem.caption := section;

      dbkf.menu_sections.Add(newitem);

    end else // if copy(s,1,1) = '[' then... (is a new section)
    begin

      // 0=caption, 1=definition, 2 ... n = values
      tokenlist( s,': ,()',sl );

      if stricomp( pchar(sl[1]), 'text' ) = 0 then
      begin

        newlabel := tlabel.Create( self );
        newlabel.left := left;
        newlabel.top  := top;
        newlabel.caption := sl[0];
        newlabel.parent := newscrollbox;

        newedit := tedit.Create(self);
        newedit.Left := left + hoffset;
        newedit.Top := top;
        newedit.Name := 'edit_'+section+'_'+sl[0];
        newedit.Parent := newscrollbox;

      end; // text

      //
      // some directory entries require double quotes at both ends
      // and a backslash. snapdir is a problem folder when there are
      // spaces (like in c:\program files) in the path name
      // The saving procedure shall have special treatment for
      // the quoted_dir items.
      //
      if stricomp( pchar(sl[1]), 'quoted_dir' ) = 0 then
      begin

        newlabel := tlabel.Create( self );
        newlabel.left := left;
        newlabel.top  := top;
        newlabel.caption := sl[0];
        newlabel.parent := newscrollbox;

        newedit := tedit.Create(self);
        newedit.Left := left + hoffset;
        newedit.Top := top;
        newedit.Name := 'quoted_dir_'+section+'_'+sl[0];
        newedit.Parent := newscrollbox;

      end; // quoted_dir


      if stricomp( pchar(sl[1]), 'spin' ) = 0 then
      begin

        newlabel := tlabel.Create( self );
        newlabel.left := left;
        newlabel.top  := top;
        newlabel.caption := sl[0];
        newlabel.parent := newscrollbox;

        newspinedit := tjvspinedit.Create(self);
        newspinedit.Left := left + hoffset;
        newspinedit.Top := top;
        newspinedit.Name := 'spin_'+section+'_'+sl[0];

        if( sl.Count = 2 ) then sl.Add('1');
        newspinedit.increment := strtoint( sl[2] );
        newspinedit.Parent := newscrollbox;

      end; // spin



      if stricomp( pchar(sl[1]), 'bool' ) = 0 then
      begin

        newcheckbox := tcheckbox.Create( self );
        newcheckbox.left := left + hoffset;
        newcheckbox.top  := top;
        newcheckbox.caption := sl[0];
        newcheckbox.name := 'cb_'+section+'_'+sl[0];
        newcheckbox.parent := newscrollbox;

      end; // bool

      if stricomp( pchar(sl[1]), 'select' ) = 0 then
      begin

        newlabel := tlabel.Create( self );
        newlabel.left := left;
        newlabel.top  := top;
        newlabel.caption := sl[0];
        newlabel.parent := newscrollbox;

        newcombobox := tcombobox.Create( self );
        newcombobox.left := left + hoffset;
        newcombobox.top  := top;
        newcombobox.name := 'edit_'+section+'_'+sl[0];
        newcombobox.parent := newscrollbox;

        if sl.Count > 2 then
        begin
          for i := 2 to sl.Count-1 do
          begin
            newcombobox.Items.Add(sl[i]);
          end;
        end;

      end; // select

      if stricomp( pchar(sl[1]), 'memo' ) = 0 then
      begin

        newmemo := tmemo.create( self );
        newmemo.name := 'memo_'+section+'_'+sl[0];
        newmemo.Parent := newscrollbox;
        newmemo.Align := alClient;
        newmemo.ScrollBars := ssVertical;
        newmemo.Hint := hint;

      end; // memo

    end; // NOT-Part of "if copy(s,1,1) = '[' then ..." (not a new section)

    top := top + voffset;

  end; // end while

  CloseFile(F);

  sl.free;

  btn_readconfig.Enabled := false; // Only once, please!

end;

procedure Tfrm_project_config.readdata( ini : tinifile );
var
  F, F_INI : textfile;
  configfile, S, section, c1, name : string;
  sl : tstringlist;

begin

  sl :=TStringList.Create;

  configfile :=
    nolastbackslash(frm_dosbox_config.lbl_mypath.Caption) +
    '\'+ frm_dosbox_config.edit_configfile.text;

  if not fileexists( configfile ) then
  begin
    MessageDlg(
      'ERROR:'+#13+#10+
      'The configuration file '+#13+#10+
      configfile+#13+#10+
      'does not exist!'+#13+#10+
      'Please get it, put it into the directory and restart me!',
      mtError, [mbOK], 0);
    exit;
  end;

  // General first tab
    section := 'general';

  s := ini.readstring( section, '#projectdir', '' );
  edit_general_projectdir.initialdir := extractfilepath( s );
  edit_general_projectdir.text := s;

  s := ini.readstring( section,'#manualfile',  '' );
  edit_general_manualfile.initialdir := extractfilepath( s );
  edit_general_manualfile.Text := s;

  s := ini.readstring( section, '#snapshotimage', '' );
  edit_general_snapshotimage.OnChange := nil;
  edit_general_snapshotimage.text := s;
  edit_general_snapshotimage.initialdir := s; //'"'+extractfilepath( s )+'"';
  //edit_general_snapshotimage.OnChange := edit_general_snapshotimagechange;

  edit_general_www.Text := ini.readstring( section, '#wwwsite', '' );

  cb_general_exit.checked := dbkf.myreadbool( ini, section, '#exit', false );
  cb_general_noconsole.checked := dbkf.myreadbool( ini, section, '#noconsole', false );

  // is a file but not a directory
  if fileexists( s ) and not directoryexists( s )
  then
  begin
    dbkf.image1.Picture.LoadFromFile( s );
  end else
  begin
    dbkf.image1.Picture.assign( dbkf.image2.Picture );
    dbkf.image1.update;
  end;

  // now come the others!

  AssignFile(F, configfile); { Datei in Dialogfeld ausgewhlt }
  Reset(F);

  while not eof( f ) do
  begin

    Readln(F, S);

    S := trim( S );

    if length( S ) = 0 then continue;

    c1 := copy( S, 1, 1 ); // firstchar

    if (c1 = '#') or (c1 = ';') then continue;

    if c1 = '[' then
    begin

      top := 0;
      section := trim( copy( s, 2, length( s )- 2  ) );

    end else // if copy(s,1,1) = '[' then... (is a new section)
    begin

      // 0=caption, 1=definition, 2 ... n = values
      tokenlist( s,': ,()',sl );

      if stricomp( pchar(sl[1]), 'text' ) = 0 then
      begin

        Name := 'edit_'+section+'_'+sl[0];
        tedit(self.FindComponent( name )).Text := ini.ReadString(section,sl[0],'');

      end; // text

      if stricomp( pchar(sl[1]), 'quoted_dir' ) = 0 then
      begin

        Name := 'quoted_dir_'+section+'_'+sl[0];
        tedit(self.FindComponent( name )).Text := ini.ReadString(section,sl[0],'');

      end; // quoted_dir

      if stricomp( pchar(sl[1]), 'spin' ) = 0 then
      begin

        Name := 'spin_'+section+'_'+sl[0];
        tjvspinedit(self.FindComponent( name )).Text := ini.ReadString(section,sl[0],'');

      end; // spin


      if stricomp( pchar(sl[1]), 'bool' ) = 0 then
      begin

        name := 'cb_'+section+'_'+sl[0];
        tcheckbox(self.FindComponent( name )).Checked :=
          dbkf.myreadbool(ini,section,sl[0],false);

      end; // bool

      if stricomp( pchar(sl[1]), 'select' ) = 0 then
      begin

        name := 'edit_'+section+'_'+sl[0];
        tedit(self.FindComponent( name )).Text := ini.ReadString(section,sl[0],'');

      end; // select

      if stricomp( pchar(sl[1]), 'memo' ) = 0 then
      begin

        name := 'memo_'+section+'_'+sl[0];

        tmemo(self.FindComponent( name )).lines.clear;

        AssignFile(f_ini, ini.FileName);
        Reset(f_ini);

        //
        // Lies bis [section]
        //

        while (s <> '['+section+']') and not eof(f_ini) do
        begin
          readln( f_ini, s ); // skip lines
        end;

        tmemo(self.FindComponent( name )).ReadOnly := true;

        while not eof(f_ini) do
        begin
          readln( f_ini, s );
          if copy( trim( s ),1,1 ) = '[' then break else
          tmemo(self.FindComponent( name )).lines.Add(s);
        end;
        CloseFile(f_ini);

        tmemo(self.FindComponent( name )).readonly := false;

      end; // memo

    end; //


  end;

  closefile( f );

  sl.free;

end;

procedure Tfrm_project_config.btn_saveprojectClick(Sender: TObject);
var
  F, F_INI : textfile;
  configfile, S, section, c1, name, filename, qd : string;
  sl : tstringlist;
  ini : tinifile;
  ok : boolean;
  i : integer;

begin

  ok := true;

  {*
   * check: does the projectname contain spaces?
   * if so, snaps will not go to the desired directory.
   *}
  if( pos (' ', edit_general_projectname.text ) > 0 ) then
  begin
    MessageDlg(
      'ERROR:'+#13+#10+
      'The projectname "'+edit_general_projectname.text+'" contains spaces.'+#13+#10+
      'Please consider using underscores _ instead of spaces.'+#13+#10+
      'Nothing is saved.'+#13+#10+
      'Sorry.', mtError, [mbOK], 0);
    ok := false;
  end;


  if( length( trim( edit_general_projectname.text ) ) = 0 ) then
  begin
    MessageDlg('ERROR:'+#13+#10+'No Projectname is given.'+#13+#10+'Nothing is saved.'+#13+#10+'Sorry.', mtError, [mbOK], 0);
    ok := false;
  end;

  if( frm_dosbox_config.cb_autosnapdir.checked ) and ok then
  begin
    // Create the path for the snaps, if it does not yet exist.
    if not directoryexists( tedit( findcomponent('edit_render_snapdir')).Text ) then
    begin

      CreateDir(tedit( findcomponent('edit_render_snapdir')).Text);

    end;

    edit_general_snapshotimage.initialdir := tedit( findcomponent('edit_render_snapdir')).Text;

  end;

  if ok = true then
  begin

    filename := mypath + edit_general_projectname.text + '.ini';
    ini := tinifile.create( filename );

    {*
     * section general = = = = = = =
     *}
    section := 'general';

    ini.writestring( section, '#projectdir', edit_general_projectdir.Text );
    ini.writestring( section, '#snapshotimage', edit_general_snapshotimage.text );
    dbkf.mywritebool( ini, section, '#exit', cb_general_exit.checked );
    dbkf.mywritebool( ini, section, '#noconsole', cb_general_noconsole.Checked );
    ini.writestring( section, '#manualfile', edit_general_manualfile.text );
    ini.WriteString( section, '#wwwsite', edit_general_www.Text );

    configfile :=
      nolastbackslash(frm_dosbox_config.lbl_mypath.Caption) +
      '\'+ frm_dosbox_config.edit_configfile.text;

    if not fileexists( configfile ) then
    begin
      MessageDlg(
        'ERROR:'+#13+#10+
        'The configuration file '+#13+#10+
        configfile+#13+#10+
        'does not exist!'+#13+#10+
        'Please get it, put it into the directory and restart me!',
        mtError, [mbOK], 0);
      exit;
    end;

    // now come the others!

    AssignFile(F, configfile); { Datei in Dialogfeld ausgewhlt }
    Reset(F);

    sl := tstringlist.create;

    while not eof( f ) do
    begin

      Readln(F, S);

      S := trim( S );

      if length( S ) = 0 then continue;

      c1 := copy( S, 1, 1 ); // firstchar

      if (c1 = '#') or (c1 = ';') then continue;

      if c1 = '[' then
      begin

        section := trim( copy( s, 2, length( s )- 2  ) );

      end else // if copy(s,1,1) = '[' then... (is a new section)
      begin

        // 0=caption, 1=definition, 2 ... n = values
        tokenlist( s,': ,()',sl );

        if stricomp( pchar(sl[1]), 'text' ) = 0 then
        begin

          Name := 'edit_'+section+'_'+sl[0];
          ini.WriteString(section,sl[0],tedit(self.FindComponent( name )).Text);

        end; // text

        if stricomp( pchar(sl[1]), 'quoted_dir' ) = 0 then
        begin

          Name := 'quoted_dir_'+section+'_'+sl[0];
          qd := str_quote_dir(tedit(self.FindComponent( name )).Text);
          ini.WriteString(section,sl[0],qd);

        end; // quoted_dir

        if stricomp( pchar(sl[1]), 'spin' ) = 0 then
        begin

          Name := 'spin_'+section+'_'+sl[0];
          ini.WriteString(section,sl[0],tjvspinedit(self.FindComponent( name )).Text);

        end; // spin


        if stricomp( pchar(sl[1]), 'bool' ) = 0 then
        begin

          name := 'cb_'+section+'_'+sl[0];
          dbkf.mywritebool(ini,section,sl[0],tcheckbox(self.FindComponent( name )).Checked);

        end; // bool

        if stricomp( pchar(sl[1]), 'select' ) = 0 then
        begin

          name := 'edit_'+section+'_'+sl[0];
          ini.WriteString(section,sl[0],tedit(self.FindComponent( name )).Text);

        end; // select

        if stricomp( pchar(sl[1]), 'memo' ) = 0 then
        begin

          name := 'memo_'+section+'_'+sl[0];
          ini.EraseSection(section); //

          ini.Free; // Write all ini-Data

          assignfile( f_ini, filename );
          append( f_ini );
          writeln( f_ini, '['+section+']');

          for i := 0 to tmemo( self.findcomponent(name)).lines.count-1 do
          begin
            writeln( f_ini, tmemo( self.findcomponent(name)).lines[i] );
          end;

          flush( f_ini );
          closefile( f_ini );

          // re-open ini file for the rest
          ini := tinifile.create( filename );

        end; // memo

     end; // not a section

    end; // while not eof ( f )

    closefile( f );

    sl.free;
    ini.free;

  end; // if ( ok )

  dbkf.readprojects;

end;

procedure Tfrm_project_config.edit_general_projectnameChange(
  Sender: TObject);
var
  snapdir : string;
  t : tedit;
begin
  if( frm_dosbox_config.cb_autosnapdir.checked ) then
  begin

    snapdir := nolastbackslash(mypath) + '\snaps\'+edit_general_Projectname.text;
    t := tedit( self.findcomponent( 'edit_render_snapdir' ) );
    t.text := snapdir;

    edit_general_snapshotimage.initialdir := snapdir;
    edit_general_snapshotimage.text := snapdir;

  end;
end;

procedure Tfrm_project_config.lbl_projectpathClick(Sender: TObject);
var
  s : string;
begin

  s := 'explorer.exe '+ extractfilepath( edit_general_projectdir.edittext )+#0;
	winexec( @s[1], SW_SHOWNORMAL );
end;

procedure Tfrm_project_config.lbl_manualfileClick(Sender: TObject);
begin
  dbkf.startprogram( edit_general_manualfile.text );
end;

procedure Tfrm_project_config.lbl_wwwClick(Sender: TObject);
begin
  dbkf.startprogram( edit_general_www.text );
end;

function tfrm_project_config.str_quote_dir( s : string ) : string;
var
  w,c : string;
begin
  w := s;

  // is first char a double quote?
  c := copy(s,1,1);
  if strcomp( '"', pchar( c ) ) = 0 then
  begin
    w := copy( s, 2, length( s ) - 2 );
  end;

  w := nolastbackslash ( w );

  result := '"' + w + '\"';

end;


end.
